#include "freq_waveform.h"

static gdouble wavetable[WAVETABLE_LENGTH]; /* possibly change this to pointer */

void generate_wavetable()
{
  gint i;
  gdouble k=2*PI;

  k/=WAVETABLE_LENGTH;

  for(i=0;i<WAVETABLE_LENGTH;++i)
    {
      wavetable[i]=sin(k*i);
    }
}

/* get sin using wavetable */ 

gdouble w_sin(glong i, gdouble k)
{
  glong offset=i*k/LOWEST_PERCEIVABLE_FREQ;

  offset%=WAVETABLE_LENGTH;
  
  return(wavetable[offset]);
}

gint half_cosine_interpolate(glong x1, glong y1, glong x2, glong y2, glong i)
{
  gdouble k;

  if(y1>y2)
    {
      k=PI/(x2-x1);
      return( y2+(y1-y2)*( cos(k*(i-x1))+1)/2 );
    }
  else if(y1<y2)
    {
      k=PI/(x2-x1);
      return(y1+(y2-y1)*(cos( k*(i-x1) + PI) +1)/2);
    }
  return(y1);
}

gint linear_interpolate(glong x1, glong y1, glong x2, glong y2, glong i)
{
  return( ( (gdouble) (y2-y1) / (x2-x1) ) * (i-x1)  + y1);
}

void free_sample(SAMPLE *samp)
{
  if(samp==NULL)
    return;
  free(samp->data);
}

void free_nodes(GList **nodes)
{
  GList *temp_node;
  COORDINATES *temp_coord;
  long length, i;

  if(nodes==NULL)
    return;
  if(*nodes==NULL)
    {
      free(*nodes);
      return;
    }
  length=g_list_length(*nodes);
  for(i=0;i<length;++i)
    {
      temp_node=g_list_nth(*nodes, i);
      temp_coord=(COORDINATES *) temp_node->data;
      free(temp_coord);
    } 
  while(length>0)
    {
      temp_node=g_list_nth(*nodes, 0);
      *nodes=g_list_remove_link(*nodes, temp_node);
      length=g_list_length(*nodes);
    }
  free(*nodes);
}

void free_waveform_glist(GList *waveforms)
{
  glong length, i;
  GList *temp_glist, *temp_waves;
  FREQ_AMPLITUDE_ENTRY *wave;

  if(waveforms==NULL)
    return;

  length=g_list_length(waveforms);

  for(i=0;i<length;++i)
    {
      temp_glist=g_list_nth(waveforms, i);
      wave=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;

      /* there's a problem here, I have to go through the whole program and make sure
	 there are no instances where I pass a constant string as the title, in the 
	 mean time, the title won't be freed like it should be */

      //      if(wave->title!=NULL)
      //	  free(wave->title);

      if(wave->samp!=NULL)
	  free_sample(wave->samp);

      if(wave->nodes!=NULL)
	free_nodes(wave->nodes);
    }
  free(wave);

  temp_waves=waveforms;
  while(length>0)
    {
      temp_glist=g_list_nth(temp_waves, 0);
      temp_waves=g_list_remove_link(temp_waves, temp_glist); 
      length=g_list_length(temp_waves);
    }
}

glong max(glong a, glong b)
{
  if(a>b)
    return(a);
  return(b);
}

gint output_as_wav(SAMPLE *a_sample, char *filename)
{
  AFfilesetup af_file;
  AFfilehandle af_handle;
  af_file=afNewFileSetup();
  afInitFileFormat(af_file, AF_FILE_WAVE);        
  afInitChannels(af_file, AF_DEFAULT_TRACK, 1);
  afInitSampleFormat(af_file, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
  af_handle=afOpenFile(filename, "w", af_file);
  afWriteFrames(af_handle, AF_DEFAULT_TRACK, a_sample->data, a_sample->length);
  afCloseFile(af_handle);
  return(TRUE);
}

glong largest_xpos_in_entries(GList *waveforms)
{
  glong largest, length, i;
  GList *temp_glist;
  FREQ_AMPLITUDE_ENTRY *temp_waveform;

  length=g_list_length(waveforms);

  if(length==0)
    return(-1);

  temp_glist=g_list_nth(waveforms, 0);
  temp_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  largest=temp_waveform->sample_length;

  for(i=1;i<length;++i)
    {
      temp_glist=g_list_nth(waveforms, 0);
      temp_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
     
      if(temp_waveform->sample_length>largest)
	  largest=temp_waveform->sample_length;
    }
  return(largest);
}

/* calculate the values on the waveform calculated from the mouse click location */

COORDINATES mouse_coords_to_wave_coords(x, y, sample_length, window_width, window_height)
     gint x, y;
     long sample_length, window_width, window_height;
{
  COORDINATES ret;
  gdouble x_coefficient, y_coefficient;
  x_coefficient=(gdouble) sample_length/window_width;
  y_coefficient=(gdouble) MAX_AMPLITUDE/window_height;
  ret.x_pos=x*x_coefficient;
  ret.amplitude=(-y+window_height)*y_coefficient;
  return(ret);
}

/*construct an array of points to be graphed in the GUI relative to the dimensions of */
/* the graph editing window  */

gint count_wave_nodes(GList *waveforms)
{
  glong i, j=0, length;
  GList *temp_glist;
  FREQ_AMPLITUDE_ENTRY *temp_waveform;

  length=g_list_length(waveforms);

  for(i=0;i<length;++i)
    {
      temp_glist=g_list_nth(waveforms, i);
      temp_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
      if(temp_waveform->type==TYPE_WAVE && temp_waveform->include_in_output==TRUE)
	++j;
    }
  return(j);
}

GRAPH_POINTS get_graph_points(FREQ_AMPLITUDE_ENTRY *waveform, long window_width, long window_height)
{
  GRAPH_POINTS ret;
  GList *list;
  COORDINATES *temp_coords;
  GList *current_entry;
  gint list_length, i;
  glong max_xpos;
  gdouble x_coefficient,  y_coefficient;

  list=*(waveform->nodes);
  list_length=g_list_length(list);
  max_xpos=largest_xpos(waveform->nodes);
  ret.length=list_length;
  ret.coords=(COORDINATES *) calloc(list_length, sizeof(struct COORDINATES));
  x_coefficient=(double) window_width/max_xpos;
  y_coefficient=(double) -window_height/32500;

  for(i=0;i<list_length;++i)
    {
      current_entry=g_list_nth(list, i);
      temp_coords=(COORDINATES *) current_entry->data;
      ret.coords[i]=*temp_coords;
      ret.coords[i].x_pos*=x_coefficient;
       ret.coords[i].amplitude=ret.coords[i].amplitude*y_coefficient+window_height;
    }
  return(ret);
}

void amplify_sample(SAMPLE *x, double coefficient)
{
  glong i;

  for(i=0;i<x->length;++i)
    x->data[i]*=coefficient;
}

/* rescale all the xpos's relative to the new maximum length */

void redo_xpos(FREQ_AMPLITUDE_ENTRY *waveform, long length)
{
  GList *temp_list;
  COORDINATES *current_coords;
  gint number_of_nodes, i;
  glong old_length;
  gdouble coefficient;

  number_of_nodes=g_list_length(*(waveform->nodes));
  temp_list=*(waveform->nodes);
  old_length=largest_xpos(waveform->nodes);
  coefficient=(gdouble) length/old_length;

  for(i=0;i<number_of_nodes;++i)
    {
      temp_list=g_list_nth(*(waveform->nodes), i);
      current_coords=(COORDINATES *) temp_list->data;
      current_coords->x_pos*=coefficient;
    }
}

void print_list(GList **header)
{
  gint length, i;
  GList *temp_list;
  COORDINATES *coords;

  if(header==NULL)
    return;

  length=g_list_length(*header);

  for(i=0;i<length;++i)
    {
      temp_list=g_list_nth(*header, i);
      coords=(COORDINATES *) temp_list->data;
      printf("%ld %ld\n", coords->x_pos, coords->amplitude);
    }
}

long largest_xpos(GList **header)
{
  GList *temp_list;
  COORDINATES *coords;

  if(*header==NULL)
    return(0);

  temp_list=g_list_last(*header);
  coords=(COORDINATES *) temp_list->data;
  
  // I'm going to assume the last node is also the largest
  return(coords->x_pos);
}

glong find_highest_value(glong *data, glong length)
{
  glong i, highest=0;

  for(i=0;i<length;++i)
    if(labs(data[i])>highest)
      highest=labs(data[i]);
  return(highest);
}

short int *convert_long_to_int(glong *data, glong length)
{
  short int *ret;
  double coefficient=1;
  glong i, high;

  high=find_highest_value(data, length);

  if(high>MAX_AMPLITUDE)
    coefficient=(double) MAX_AMPLITUDE/find_highest_value(data, length);

  ret=(short int *) calloc(length, sizeof(short int));
  memset(ret, 0, length * sizeof(short int));

  high=find_highest_value(data, length);
  for(i=0;i<length;++i)
    {
      if(data[i]>high)
	printf("Too high %ld %ld\n", i, data[i]);
      ret[i]=coefficient * data[i];
      if(coefficient * *(data+i)>32500 || coefficient * *(data+i) <-32500 )
	printf("%ld %f\n", i, coefficient * data[i] );
    }
  free(data);
  return(ret);
}

SAMPLE *build_sample(FREQ_AMPLITUDE_ENTRY *waveform, gint interpolation_mode)
{
  SAMPLE *ret;
  short int *wav_buffer;
  COORDINATES *p1, *p2;
  glong i, j, length=0, v, list_length=0;
  GList *temp_list1, *temp_list2, **header;

  header=waveform->nodes;

  ret=(SAMPLE *) calloc(1, sizeof(struct SAMPLE));

  if(*header==NULL)
    return((SAMPLE *) NULL);

  list_length=g_list_length(*header);

  if(list_length==1)
    return((SAMPLE *) NULL);

  length=largest_xpos(header);
  wav_buffer=(short int *) calloc(length+2, sizeof(short int));
  memset(wav_buffer, 0, length*sizeof(short int));

  list_length=g_list_length(*header);

  temp_list1=g_list_nth(*header, 0);
  p1=(COORDINATES *) temp_list1->data;

  for(j=1;j<list_length;++j)
    {
      temp_list2=g_list_nth(*header, j);
      p2=(COORDINATES *) temp_list2->data;

      if(p1->x_pos<p2->x_pos && p1->x_pos>=0) // sanity check
	{
	  for(i=p1->x_pos;i<=p2->x_pos;++i)
	    {

	      switch(interpolation_mode)
		{
		case(I_LINEAR):
		  v=linear_interpolate(p1->x_pos, p1->amplitude, p2->x_pos, p2->amplitude, i);
		  break;
		case(I_HALF_COS):
		  v=half_cosine_interpolate(p1->x_pos, p1->amplitude, p2->x_pos, p2->amplitude, i);
		  break;
		}
	      wav_buffer[i]=(short int) v*w_sin(i, waveform->freq);
	      if(v>32500)
		printf("v=%ld \n", v);
	    }
	}
      p1=p2;
    }
  ret->data=wav_buffer;
  ret->length=length;

  return(ret);
}

glong maximum_length_of_sample(SAMPLE **sample_array, int number_of_samples)
{
  glong ret, i;

  if(sample_array==NULL)
    return(0);
  if(*sample_array==NULL)
    return(0);

  ret=sample_array[0]->length;

  for(i=1;i<number_of_samples;++i)
    if(sample_array[i]->length>ret)
      ret=sample_array[i]->length;
  return(ret);
}

SAMPLE *combine_samples(SAMPLE **sample_array, gint number_of_samples)
{
  glong i, j;
  SAMPLE *ret;
  glong *finished_sample;
  glong length;

  length=maximum_length_of_sample(sample_array, number_of_samples);

  finished_sample=(glong *) calloc(length+2, sizeof(glong));
  memset(finished_sample, 0, length*sizeof(glong));
  ret=(SAMPLE *) calloc(1, sizeof(struct SAMPLE));

  if(finished_sample==NULL)
    printf("looped_sample==NULL");

  memset(finished_sample, 0, length*sizeof(glong));

  for(j=0;j<number_of_samples;++j)
    {
      for(i=0;i<sample_array[j]->length;++i)
	{
	  finished_sample[i]+=*(sample_array[j]->data + i);
	}
    }

  ret->data=convert_long_to_int(finished_sample, length);
  ret->length=length;
  return(ret);
}

LINE find_current_list_index(GList *list, glong x)
{
  LINE ret_line;
  glong current_index=0, length;
  GList *list1, *list2;
  COORDINATES *p1, *p2;

 if(list==NULL)
   {
     printf("list==NULL\n");
      ret_line.m=0;
      ret_line.b=0;
      return(ret_line);
   }

  list1=g_list_last(list);
  p1=(COORDINATES *) list1->data;

  if(p1->x_pos<x)
    {
     printf("p1->x_pos=%ld x=%ld\n", p1->x_pos, x);
      ret_line.m=0;
      ret_line.b=0;
      return(ret_line);
    }

  length=g_list_length(list);

  /* this is basically a sanity check, all node lists must have 2 nodes */

  if(length<2)
    {
      ret_line.m=0;
      ret_line.b=0;
      return(ret_line);
    }

  /* if the function has made it this far, x must be between two nodes */

  list1=g_list_nth(list, current_index);
  list2=g_list_nth(list, current_index+1);
  p1=(COORDINATES *) list1->data;
  p2=(COORDINATES *) list2->data;
  current_index++;

  while(p2->x_pos<x)
    {
      current_index++;
      list1=list2;
      p1=p2;
      list2=g_list_nth(list, current_index);
      p2=(COORDINATES *) list2->data;
    }
 
  ret_line.m=(gdouble) (p2->amplitude-p1->amplitude)/(p2->x_pos-p1->x_pos);
  ret_line.b=p1->amplitude;
  return(ret_line);
}

glong calculate_harmonics(glong n_sample, glong c, glong d, glong a, glong b)
{
  glong ret=0;
  glong theta=n_sample%SAMPLES_PER_SEC;
  glong i, high, low;
  gdouble k, m, sample_coefficient;

  if(c==d)
    return(0);

  m=(gdouble) (a-b)/(c-d);

  if(c>d)
    {
      high=c;
      low=d;
    }
  else
    {
      high=d;
      low=c;
    }

  for(i=1;i<high-low;++i)
    {
      k=(gdouble) m*i+a;
      sample_coefficient=(gdouble) (2*PI*(i+low))/SAMPLES_PER_SEC;
      ret+=k*sin(theta*(i+low)*sample_coefficient);
    }
  ret/=high-low;
  return(ret);
}

/* this is an experimental section that interpolates frequencies that aren't explictly
   specified by a frequency amplitude waveform. For instance, if I have one waveform
   at 1000 Hz and another at 1002 Hz, the program would interpolate a waveform for 
   1001 Hz. At present this does not work in any useful sense and it's incredibly slow */

SAMPLE *add_harmonic_sample(FREQ_AMPLITUDE_ENTRY *a, FREQ_AMPLITUDE_ENTRY *b)
{
  SAMPLE *ret;
  glong *wav_buffer;
  glong i, length=0, c1, c2, harmonic_sum;
  GList **header_1, **header_2;
  LINE line1, line2;

  header_1=a->nodes;
  header_2=b->nodes;

  ret=(SAMPLE *) calloc(1, sizeof(struct SAMPLE));

  if(*header_1==NULL || *header_2==NULL)
    return((SAMPLE *) NULL);

  length=max(largest_xpos(header_1), largest_xpos(header_2));
  wav_buffer=(glong *) calloc(length+2, sizeof(glong));
  memset(wav_buffer, 0, length*sizeof(glong));

  for(i=0;i<length;++i)
    {
      line1=find_current_list_index(*header_1, i);
      line2=find_current_list_index(*header_2, i);

      c1=line1.m*i+line1.b;
      c2=line2.m*i+line2.b;

      harmonic_sum=calculate_harmonics(i, a->freq, b->freq, c1, c2);
      *(wav_buffer+i)=(glong) harmonic_sum;
    }

  ret->data=convert_long_to_int(wav_buffer, length);
  ret->length=length;
  play_sample(ret);
  return(ret);
}

SAMPLE *create_sound(GList *waveforms, int interpolation_method)
{
  SAMPLE **sample, *final_sample, *temp_sample;
  glong length, i, j=0, n_waves, sample_length;
  GList *temp_glist1;
  FREQ_AMPLITUDE_ENTRY *temp_waveform1;

  n_waves=count_wave_nodes(waveforms);
  length=g_list_length(waveforms);

  if(n_waves==0 || length==0)
    return(NULL);

  // we need a sample for each frequency defined, and n-1 samples for
  // the frequencies in between

  sample=(SAMPLE **) calloc(n_waves, sizeof(struct SAMPLE *));
  sample_length=largest_xpos_in_entries(waveforms);

  if(sample_length<100)
    sample_length=BUFFER_LENGTH;

  for(i=0;i<length;++i)
    {
      temp_glist1=g_list_nth(waveforms, i);
      temp_waveform1=(FREQ_AMPLITUDE_ENTRY *) temp_glist1->data;
      if(temp_waveform1->include_in_output==TRUE)
	{
	  *(sample+j)=build_sample(temp_waveform1, interpolation_method);
	  amplify_sample(*(sample+j), temp_waveform1->mixing_coefficient);
	  ++j;
	}
    }

  /* the harmonics section is depreciated for now */

/*    if(include_harmonics==TRUE) */
/*      { */
/*        for(i=1;i<length;++i) */
/*  	{ */
/*  	  temp_glist1=g_list_nth(waveforms, i-1); */
/*  	  temp_glist2=g_list_nth(waveforms, i); */

/*  	  temp_waveform1=(FREQ_AMPLITUDE_ENTRY *) temp_glist1->data; */
/*  	  temp_waveform2=(FREQ_AMPLITUDE_ENTRY *) temp_glist2->data; */

/*  	  if(temp_waveform1->include_in_output==TRUE &&  */
/*  	     temp_waveform2->include_in_output==TRUE) */
/*  	    { */
/*  	      printf("Adding harmonic %ld %ld\n", temp_waveform1->freq,  */
/*  		 temp_waveform2->freq); */
/*  	      sample[j]=add_harmonic_sample(temp_waveform1, temp_waveform2); */
/*  	      ++j; */
/*  	    } */
/*  	} */
  //    }

  final_sample=combine_samples(sample, j);

  for(i=0;i<n_waves;++i)
    {
      temp_sample=(SAMPLE *) *(sample+i);
      free(temp_sample->data);
    }
  return(final_sample);
}

gint does_xpos_exist(GList **header, long x)
{
  GList *temp_list;
  COORDINATES *coords;
  glong length, i;

  length=g_list_length(*header);

  for(i=0;i<length;++i)
    {
      temp_list=g_list_nth(*header, i);
      coords=(COORDINATES *) temp_list->data;
      if(coords->x_pos==x)
	return(TRUE);
    }
  return(FALSE);

}

gint delete_xpos(GList **header, long x)
{
  GList *temp_list;
  COORDINATES *coords;
  glong length, i, lowest_dist, lowest_index;

  length=g_list_length(*header);

  if(length<=2)
    return(FALSE);

  /* make it impossible to delete the first or last node by skipping them */

  temp_list=g_list_nth(*header, 1);
  coords=(COORDINATES *) temp_list->data;

  lowest_dist=labs(coords->x_pos-x);
  lowest_index=1;

  for(i=2;i<length-1;++i)
    {
      temp_list=g_list_nth(*header, i);
      coords=(COORDINATES *) temp_list->data;
      if(labs(coords->x_pos-x)<lowest_dist)
	{
	  lowest_index=i;
	  lowest_dist=labs(coords->x_pos-x);
	}
    }

  temp_list=g_list_nth(*header, lowest_index);
  free(temp_list->data);
  *header=g_list_remove_link(*header, temp_list);

  return(TRUE);
}

gint play_sample(SAMPLE *samp)
{
  int fd, arg;

  if( (fd=open("/dev/dsp", O_WRONLY))<0)
    {
      return(-1);
    }

  arg=0;
  ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &arg);

  arg=0;
  ioctl(fd, SNDCTL_DSP_STEREO, &arg);
 
  arg=44100;
  ioctl(fd, SOUND_PCM_WRITE_RATE, &arg);

  arg=44100;
  ioctl(fd, SNDCTL_DSP_SPEED, &arg);

  arg=AFMT_S16_NE;
  ioctl(fd, SNDCTL_DSP_SETFMT, &arg);

  write(fd, samp->data, samp->length*sizeof(short int));
  close(fd);
  return(0);
}

void append_node(GList **n, COORDINATES x)
{
  COORDINATES *ptr;
 
  ptr=(COORDINATES *) g_malloc(sizeof(struct COORDINATES));
  ptr->x_pos=x.x_pos;
  ptr->amplitude=x.amplitude;
  *n=g_list_append(*n, (gpointer) ptr);
}

void add_node(GList **n, COORDINATES x)
{
  gpointer *ptr;
  GList *temp1, *temp2;
  COORDINATES *coord1, *coord2;
  gint length, i;

  ptr=g_malloc(sizeof(struct COORDINATES));
  memcpy(ptr, &x, sizeof(struct COORDINATES));

  if(*n==NULL)
    {
      *n=g_list_append(*n, ptr);
      return;
    }

  length=g_list_length(*n);
  temp1=g_list_nth(*n, 0);
  coord1=(COORDINATES *) temp1->data;
  for(i=1;i<length;++i)
    {
      temp2=g_list_nth(*n, i);
      coord2=(COORDINATES *) temp2->data;
      if(coord1->x_pos<x.x_pos && coord2->x_pos>x.x_pos)
	{
	  *n=g_list_insert(*n, ptr, i);
	  return;
	}
      coord1=coord2;
    }
  *n=g_list_append(*n, ptr);
}

GList **create_basic_waveform(length)
     long length;
{
  GList **ret;
  COORDINATES a_value;

  ret=(GList **) calloc(1, sizeof(GList *));
  *ret=NULL;

  a_value.x_pos=0;
  a_value.amplitude=0;
  append_node(ret, a_value);
  a_value.x_pos=length;
  a_value.amplitude=0;
  append_node(ret, a_value);
  return(ret);
}

FREQ_AMPLITUDE_ENTRY *create_new_waveform(char *title, int type, long length, double mixing, glong freq)
{
  FREQ_AMPLITUDE_ENTRY *ret;

  ret=(FREQ_AMPLITUDE_ENTRY *) calloc(1, sizeof(FREQ_AMPLITUDE_ENTRY));
  ret->type=type;
  ret->sample_length=length;
  ret->include_in_output=TRUE;
  ret->title=title;
  ret->mixing_coefficient=mixing;
  ret->freq=freq;
  ret->samp=NULL;
  ret->nodes=create_basic_waveform(length);
  return(ret);
} 

FREQ_AMPLITUDE_ENTRY *copy_waveform(FREQ_AMPLITUDE_ENTRY *src)
{
  FREQ_AMPLITUDE_ENTRY *ret;
  gchar *title;
  gint title_length;
  glong i, n_nodes;
  GList *temp_glist;
  COORDINATES *temp_coords;

  if(src->title!=NULL)
    {
      title_length=strlen(src->title);
      title=(gchar *) calloc(title_length, sizeof(gchar));
      strncpy(title, src->title, title_length);
    }
  else
    title=NULL;

  ret=create_new_waveform(title, src->type, src->sample_length, src->mixing_coefficient,
			  src->freq);

  /* now copy nodes */
  n_nodes=g_list_length(*(src->nodes));

  for(i=1;i<n_nodes-1;++i)
    {
      temp_glist=g_list_nth(*(src->nodes), i);
      temp_coords=(COORDINATES *) temp_glist->data;
      add_node(ret->nodes, *temp_coords);
    }
  return(ret);
}

gint save_wave(char *filename, GList *waveforms)
{
  FILE *output_file;
  FREQ_AMPLITUDE_ENTRY *temp_waveform;
  COORDINATES *temp_coords;
  GList *temp1_glist, *temp2_glist;

  long number_of_waveforms, number_of_nodes, i, j;

  if( (output_file=fopen(filename, "wt"))==NULL)
    {
      g_message("error writing to file\n");
      return(FALSE);
    }

  number_of_waveforms=g_list_length(waveforms);
  fprintf(output_file, "%ld\n", number_of_waveforms);

  for(i=0;i<number_of_waveforms;++i)
    {
      temp1_glist=g_list_nth(waveforms, i);
      temp_waveform=(FREQ_AMPLITUDE_ENTRY *) temp1_glist->data;

      fprintf(output_file, "%s\n", temp_waveform->title);
      fprintf(output_file, "%d\n", temp_waveform->type);
      fprintf(output_file, "%d\n", temp_waveform->include_in_output);
      fprintf(output_file, "%g\n", temp_waveform->mixing_coefficient);
      fprintf(output_file, "%ld\n", temp_waveform->sample_length);
      fprintf(output_file, "%ld\n", temp_waveform->freq);

      temp1_glist=*temp_waveform->nodes;
      number_of_nodes=g_list_length(temp1_glist);
      fprintf(output_file, "%ld\n", number_of_nodes);

      for(j=0;j<number_of_nodes;++j)
	{
	  temp2_glist=g_list_nth(temp1_glist, j);
	  temp_coords=(COORDINATES *) temp2_glist->data;
	  fprintf(output_file, "%ld %ld\n", temp_coords->x_pos, 
		  temp_coords->amplitude);
	}
    }
  fclose(output_file);
  return(TRUE);
}

GList **read_node_list(FILE *input_file, long number_of_nodes)
{
  GList **ret, *list=NULL;
  COORDINATES *new_coord;
  long i;

  ret=g_malloc(sizeof( GList * ));

  for(i=0;i<number_of_nodes;++i)
    {
      new_coord=(COORDINATES *) g_malloc(sizeof(struct COORDINATES));
      if(fscanf(input_file, "%ld %ld", &(new_coord->x_pos), &(new_coord->amplitude))<2)
	 return((GList **) NULL);
      list=g_list_append(list, new_coord);
    }
  *ret=list;
  return(ret);
}

GList *open_wave(char *filename)
{
  GList *ret=NULL;
  FILE *input_file;
  FREQ_AMPLITUDE_ENTRY *temp_waveform;
  glong number_of_waveforms, number_of_nodes, i;
  gchar temp_title[80];
  
  if( (input_file=fopen(filename, "rt"))==NULL)
    {
      g_message("Error opening file\n");
      return(ret);
    }

  fscanf(input_file, "%ld", &number_of_waveforms);

  for(i=0;i<number_of_waveforms;++i)
    {
      temp_waveform=(FREQ_AMPLITUDE_ENTRY *) g_malloc(sizeof(FREQ_AMPLITUDE_ENTRY));
      fgets(temp_title, 80, input_file);
      fgets(temp_title, 80, input_file);
      puts(temp_title);
      temp_waveform->title=NULL;
      //      if(strlen(temp_title)>0)
      //	temp_title[strlen(temp_title)-1]='\0';
      //      temp_waveform->title=(char *) g_malloc(strlen(temp_title)+1);
      // strncpy(temp_waveform->title, temp_title, 80);
      fscanf(input_file, "%d", &temp_waveform->type);
      fscanf(input_file, "%d", &temp_waveform->include_in_output);
      fscanf(input_file, "%lf", &temp_waveform->mixing_coefficient);
      fscanf(input_file, "%ld", &temp_waveform->sample_length);
      fscanf(input_file, "%ld", &temp_waveform->freq);
      temp_waveform->samp=NULL;
 /*       printf("%s:%d:%d:%f:%ld\n", temp_waveform->title, temp_waveform->type,  */
/*  	     temp_waveform->include_in_output, temp_waveform->mixing_coefficient,  */
/*  	     temp_waveform->sample_length); */
      
      number_of_nodes=0;
      fscanf(input_file, "%ld", &number_of_nodes);
      temp_waveform->nodes=read_node_list(input_file, number_of_nodes);
      ret=g_list_append(ret, temp_waveform);
    }

  fclose(input_file);
  return(ret);
}

glong highest_amplitude_in_list(GList *waveforms)
{
  glong length1, length2, i, j, high=0;
  GList *temp_glist1, *node_list, *temp_glist2;
  FREQ_AMPLITUDE_ENTRY *temp_freq1;
  COORDINATES *temp_coord;

  length1=g_list_length(waveforms);

  for(i=0;i<length1;++i)
    {
      temp_glist1=g_list_nth(waveforms, i);
      temp_freq1=temp_glist1->data;
      node_list=*(temp_freq1->nodes);

      length2=g_list_length(node_list);
      for(j=0;j<length2;++j)
	{
	  temp_glist2=g_list_nth(node_list, j);
	  temp_coord=temp_glist2->data;

	  if(temp_coord->amplitude>high)
	    high=temp_coord->amplitude;
	}		 
    }
  return(high);
}

void normalize_nodes(GList *waveforms)
{
  glong length1, length2, i, j, high=0;
  GList *temp_glist1, *node_list, *temp_glist2;
  FREQ_AMPLITUDE_ENTRY *temp_freq1;
  COORDINATES *temp_coord;
  gdouble k=0;

  high=highest_amplitude_in_list(waveforms);

  k=(MAX_AMPLITUDE*0.8)/high;
  
  length1=g_list_length(waveforms);

  for(i=0;i<length1;++i)
    {
      temp_glist1=g_list_nth(waveforms, i);
      temp_freq1=temp_glist1->data;
      node_list=*(temp_freq1->nodes);

      length2=g_list_length(node_list);
      for(j=0;j<length2;++j)
	{
	  temp_glist2=g_list_nth(node_list, j);
	  temp_coord=temp_glist2->data;

	  temp_coord->amplitude*=k;
	}		 
    }  
}
