#include "freq_waveform.h"

static void update_drawing_area(GtkWidget *widget);
static char initial_sample_length_text[]={"44100"};
static GdkPixmap *pixmap=NULL;
int mode, include_harmonics, interpolation;
GtkWidget *window, *sample_length_entrybox, *drawing_area, *include_this_toggle, 
  *mixing_entrybox, *file_menu, *wav_menu, *freq_entrybox, *waveform_title_label;
gchar *filename, *wav_filename;
GList *waveforms;
glong waveform_index, default_length;

FREQ_AMPLITUDE_ENTRY *current_waveform;

void free_actual_waveforms_list()
{
  glong length;
  GList *temp_waves;

  length=g_list_length(waveforms);
  while(length>0)
    {
      temp_waves=g_list_nth(waveforms, 0);
      waveforms=g_list_remove_link(waveforms, temp_waves); 
      length=g_list_length(waveforms);
    }
}

void sort_waveforms()
{
  glong i, j, length;
  FREQ_AMPLITUDE_ENTRY *c, *d, *temp; 
  GList *a, *b;

  length=g_list_length(waveforms);

  for(i=0;i<length;++i)
    for(j=i;j<length;++j)
      {
	a=(GList *) g_list_nth(waveforms, i);
	b=(GList *) g_list_nth(waveforms, j);

	c=(FREQ_AMPLITUDE_ENTRY *) a->data;
	d=(FREQ_AMPLITUDE_ENTRY *) b->data;

	if(c->freq>d->freq)
	  {
	    temp=(FREQ_AMPLITUDE_ENTRY *) a->data;
	    a->data=b->data;
	    b->data=temp;
	  }
      }

  for(i=0;i<length;++i)
    {
      a=(GList *) g_list_nth(waveforms, i);
      if(a->data==current_waveform)
	{
	  waveform_index=i;
	  return;
	}
    }
}

void mouse_add_node(x, y, widget)
     gdouble x, y;
     GtkWidget *widget;
{
  COORDINATES new_coordinate;

  new_coordinate=mouse_coords_to_wave_coords((gint) x,(gint) y, 
					     current_waveform->sample_length, 
					     widget->allocation.width, 
					     widget->allocation.height);
  if(does_xpos_exist(current_waveform->nodes, new_coordinate.x_pos)==FALSE)
    add_node(current_waveform->nodes, new_coordinate);
  update_drawing_area(drawing_area);
}

void mouse_delete_node(x, y, widget)
     gdouble x, y;
     GtkWidget *widget;
{
  COORDINATES new_coordinate;
  new_coordinate=mouse_coords_to_wave_coords((gint) x,(gint) y, 
					     current_waveform->sample_length, 
					     widget->allocation.width, 
					     widget->allocation.height);

  delete_xpos(current_waveform->nodes, new_coordinate.x_pos);
}

/* this is a feature that is currently not available */

void mouse_select_node(x, y)
     gint x, y;
{

}

/* dragging isn't implemented yet */

void mouse_drag_node(x, y)
     gint x, y;
{

}

void half_cos_callback(GtkWidget *widget, gpointer data)
{
  if(interpolation==I_LINEAR)
    interpolation=I_HALF_COS;
  else
    interpolation=I_LINEAR;
}

void output_wav_ok_callback(GtkWidget *widget, GtkFileSelection *fs)
{
  SAMPLE *final_sample;
  glong n_waves;
  gchar *temp_filename;
  gint length, add_suffix=0;

  n_waves=count_wave_nodes(waveforms);
 
  if(n_waves==0)
    return;

  if( (final_sample=create_sound(waveforms, interpolation)) == NULL)
    return;

  if(wav_filename!=NULL)
    {
      free(wav_filename);
      wav_filename=NULL;
    }

  temp_filename=gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
  length=strlen(temp_filename);

  if(length<4)
    {
      add_suffix=1;
    }
  else if(strcmp(".wav", temp_filename+length-4) !=0)
	{
	  add_suffix=1;
	}

  if(add_suffix==1)
    {
      wav_filename=(gchar *) calloc(length+5, sizeof(gchar));
      sprintf(wav_filename, "%s.wav", temp_filename);
    }
  else
    {
      wav_filename=(gchar *) calloc(length+1, sizeof(gchar));
      strncpy(wav_filename, temp_filename, length);
    }
 
  output_as_wav(final_sample, wav_filename);
  free(final_sample->data);
  free(final_sample);

  gtk_widget_destroy(wav_menu);
}

void output_wav_cancel_callback(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(wav_menu);
}

void output_wav_callback(GtkWidget *widget, gpointer data)
{
  wav_menu=gtk_file_selection_new("Output as wav");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (wav_menu)->ok_button),
                    "clicked", (GtkSignalFunc) output_wav_ok_callback, wav_menu);  
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (wav_menu)->cancel_button),
		      "clicked", GTK_SIGNAL_FUNC(output_wav_cancel_callback), 
		      (gpointer) "cancel"); 
  gtk_widget_show(wav_menu);

}

void backward_button_callback(GtkWidget *widget, gpointer data)
{
  glong length;
  GList *temp_glist;

  length=g_list_length(waveforms);

  if(waveform_index==0)
    return;

  --waveform_index;
  temp_glist=g_list_nth(waveforms, waveform_index);
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(include_this_toggle), 
			       current_waveform->include_in_output);

  update_entryboxs();
  update_drawing_area(drawing_area);
}

void save_as_ok_callback(GtkWidget *widget, GtkFileSelection *fs)
{
  gchar *temp_filename;
  gint length;

  if(filename!=NULL)
    {
      free(filename);
      filename=NULL;
    }
  temp_filename=gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
  length=strlen(temp_filename);
  filename=(gchar *) calloc(length+1, sizeof(gchar));
  strncpy(filename, temp_filename, length);
  save_wave(filename, waveforms);
  gtk_widget_destroy(file_menu);
  update_entryboxs();
}

void save_as_cancel_callback(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(file_menu);
}

void save_as_callback(GtkWidget *widget, gpointer data)
{
  file_menu=gtk_file_selection_new("Save As");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_menu)->ok_button),
                        "clicked", (GtkSignalFunc) save_as_ok_callback, file_menu);  
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_menu)->cancel_button),
		      "clicked", GTK_SIGNAL_FUNC(save_as_cancel_callback), 
		      (gpointer) "cancel"); 
  gtk_widget_show(file_menu);
}

void save_callback(GtkWidget *widget, gpointer data)
{
  if(filename==NULL)
    {
      save_as_callback(widget, data);
    }
  else
    {
      save_wave(filename, waveforms);
    }
}

void open_ok_callback(GtkWidget *widget, GtkFileSelection *fs)
{
  GList *temp_glist;
  gchar *temp_filename;
  gint length;

  printf("Using filename %s\n", gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
  free_waveform_glist(waveforms);

  if(filename!=NULL)
    {
      free(filename);
      filename=NULL;
    }
  temp_filename=gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
  length=strlen(temp_filename);

  filename=(gchar *) calloc(length+1, sizeof(gchar));
  strncpy(filename, temp_filename, length);
  waveforms=NULL;
  waveforms=open_wave(filename);

  waveform_index=0;
  temp_glist=g_list_nth(waveforms, waveform_index);
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  sort_waveforms();
  update_entryboxs();
  update_drawing_area(drawing_area);
  gtk_widget_destroy(file_menu);
}

void open_cancel_callback(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(file_menu);
}

void new_callback(GtkWidget *widget, gpointer data)
{
  GList *temp_glist;

  if(filename!=NULL)
    {
      free(filename);
      filename=NULL;
    }

  free_waveform_glist(waveforms);  
  waveforms=NULL;
  add_new_waveform("Untitled 1", TYPE_WAVE, DEFAULT_SAMPLE_LENGTH, 1, DEFAULT_FREQ);
  temp_glist=g_list_nth(waveforms, 0);
  waveform_index=0;
  include_harmonics=FALSE;
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  update_entryboxs();
  update_drawing_area(drawing_area);
}

void open_callback(GtkWidget *widget, gpointer data)
{
  file_menu=gtk_file_selection_new("Open");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_menu)->ok_button),
                        "clicked", (GtkSignalFunc) open_ok_callback, file_menu);  
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_menu)->cancel_button),
		      "clicked", GTK_SIGNAL_FUNC(open_cancel_callback), 
		      (gpointer) "cancel"); 
  gtk_widget_show(file_menu);
}

void forward_button_callback(GtkWidget *widget, gpointer data)
{
  glong length;
  GList *temp_glist;

  length=g_list_length(waveforms);

  if(waveform_index>=length-1)
    return;

  ++waveform_index;
  temp_glist=g_list_nth(waveforms, waveform_index);
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(include_this_toggle), 
			       current_waveform->include_in_output);
  update_entryboxs();
  update_drawing_area(drawing_area);
}

void add_radio_button_callback(GtkWidget *widget, gpointer data)
{
    if (GTK_TOGGLE_BUTTON (widget)->active)
      {
	mode=MODE_ADD_NODE;
      } 
    else
      {
      }
}

void select_radio_button_callback (GtkWidget *widget, gpointer data)
{
    if (GTK_TOGGLE_BUTTON (widget)->active)
      {
	mode=MODE_SELECT_NODE;
      } 
    else
      {
      }
}

void delete_radio_button_callback (GtkWidget *widget, gpointer data)
{
    if (GTK_TOGGLE_BUTTON (widget)->active)
      {
	mode=MODE_DELETE_NODE;
      } 
    else
      {
      }
}

void drag_radio_button_callback(GtkWidget *widget, gpointer data)
{
    if (GTK_TOGGLE_BUTTON (widget)->active)
      {
	mode=MODE_DRAG_NODE;
      } 
    else
      {
      }
}

void normalize_nodes_callback(GtkWidget *widget, gpointer data)
{
  normalize_nodes(waveforms);
  update_drawing_area(drawing_area);
}

void copy_waveform_callback(GtkWidget *widget, gpointer data)
{
  FREQ_AMPLITUDE_ENTRY *copy;

  copy=copy_waveform(current_waveform);
  waveforms=g_list_append(waveforms, copy);
  sort_waveforms();
  update_entryboxs();
}

void set_sample_length_button_callback(GtkWidget *widget, gpointer data)
{
  gchar *entry_text;
  glong length=0;

  entry_text=gtk_entry_get_text(GTK_ENTRY(sample_length_entrybox));
  length=atol(entry_text);
  if(length<1 || length>4410000)
    return;
  redo_xpos(current_waveform, length);
  default_length=length;
  current_waveform->sample_length=length;
  update_drawing_area(drawing_area);
}

void mixing_entrybox_button_callback(GtkWidget *widget, gpointer data)
{
  gchar *entry_text;
  gdouble coefficient=0;

  entry_text=gtk_entry_get_text(GTK_ENTRY(mixing_entrybox));
  coefficient=atof(entry_text);
  if(fabs(coefficient)>100)
    return;
  current_waveform->mixing_coefficient=coefficient;
}

void freq_entrybox_button_callback(GtkWidget *widget, gpointer data)
{
  gchar *entry_text;
  glong freq=0;

  entry_text=gtk_entry_get_text(GTK_ENTRY(freq_entrybox));
  freq=atol(entry_text);
  printf("freq=%ld\n", freq);
  if(freq<0 || freq>22050)
    return;
  current_waveform->freq=freq;
  sort_waveforms();
  update_entryboxs();
}

void play_button_callback(GtkWidget *widget, gpointer data)
{
  SAMPLE *final_sample;
  glong n_waves;

  n_waves=g_list_length(waveforms);
  if(n_waves==0)
    return;

  if( (final_sample=create_sound(waveforms, interpolation)) == NULL)
    return;
  play_sample(final_sample);
  free(final_sample->data);
}

void play_only_current_button_callback(GtkWidget *widget, gpointer data)
{
  SAMPLE **sample, *final_sample;

  sample=(SAMPLE **) calloc(1, sizeof(struct SAMPLE *));
  *sample=build_sample(current_waveform, interpolation);
  amplify_sample(*sample, current_waveform->mixing_coefficient);
  final_sample=combine_samples(sample, 1);
  play_sample(final_sample);
  free((*sample)->data);
  free(final_sample->data);
}

void add_new_waveform_button_callback(GtkWidget *widget, gpointer data)
{
  GList *temp_glist;

  add_new_waveform("Untitled", TYPE_WAVE, default_length, 1, DEFAULT_FREQ);
  temp_glist=g_list_last(waveforms);
  waveform_index=g_list_length(waveforms)-1;
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(include_this_toggle), 
			       current_waveform->include_in_output);
  update_entry_text(current_waveform->sample_length);
  update_mixing_text(current_waveform->mixing_coefficient);
  update_drawing_area(drawing_area);
  sort_waveforms();
  update_entryboxs();
}

void delete_current_waveform_button_callback(GtkWidget *widget, gpointer data)
{
  GList *temp_glist;

  if(g_list_length(waveforms)<=1)
    {
      g_message("Can't delete only waveform");
      return;
    }

  free_sample(current_waveform->samp);
  free_nodes(current_waveform->nodes);
  free(current_waveform);
  temp_glist=g_list_nth(waveforms, waveform_index);
  waveforms=g_list_remove_link(waveforms, temp_glist);

  if(waveform_index!=0)
      --waveform_index;

  temp_glist=g_list_nth(waveforms, waveform_index);
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  update_entryboxs();
  update_drawing_area(drawing_area);
}

void include_this_toggle_callback(GtkWidget *widget, gpointer data)
{
  if(GTK_TOGGLE_BUTTON (widget)->active)
    {
      current_waveform->include_in_output=TRUE;
    } 
  else 
    {
      current_waveform->include_in_output=FALSE;
    }
}

gchar *waveform_title()
{
  gchar *ret, a[80];
  gint length;
  
  sprintf(a, "Waveform #%ld of %d", waveform_index+1, g_list_length(waveforms));
  length=strlen(a);
  ret=(gchar *) calloc(length+1, sizeof(gchar));
  strncpy(ret, a, length);
  return(ret);
}

static void exit_menu_item(GtkWidget *w, gpointer data)
{
  gtk_exit(0);
}

static void draw_node_lines(GtkWidget *widget)
{
  GRAPH_POINTS points;
  gint i;

  points=get_graph_points(current_waveform, widget->allocation.width, 
			  widget->allocation.height);
  for(i=1;i<points.length;++i)
    {
      gdk_draw_line(pixmap, widget->style->black_gc, points.coords[i-1].x_pos,
		    points.coords[i-1].amplitude, points.coords[i].x_pos,
		    points.coords[i].amplitude);      
    }

  for(i=0;i<points.length;++i)
    {
      gdk_draw_rectangle(pixmap, widget->style->black_gc, TRUE, 
			 points.coords[i].x_pos-2, points.coords[i].amplitude-2, 5, 5);

    }
  free(points.coords);
}

static void update_drawing_area(GtkWidget *widget)
{
  GdkRectangle update_rect;
  update_rect.x=0;
  update_rect.y=0;
  update_rect.width=widget->allocation.width;
  update_rect.height=widget->allocation.height;

  gdk_draw_rectangle(pixmap, widget->style->white_gc, TRUE, 0, 0, 
		     widget->allocation.width, widget->allocation.height);
  gdk_draw_line(pixmap, widget->style->black_gc, 0, 
		widget->allocation.height/2, widget->allocation.width, 
		widget->allocation.height/2);
  draw_node_lines(widget);
  gtk_widget_draw(widget, &update_rect);
}

static gint button_press_event(GtkWidget *widget, GdkEventButton *event)
{
  if(event->button==1 && pixmap != NULL)
    {
      switch(mode)
	{
	case(MODE_ADD_NODE):
	  mouse_add_node(event->x, event->y, widget);
	  break;
	case(MODE_DELETE_NODE):
	  mouse_delete_node(event->x, event->y, widget);
	  break;
	case(MODE_SELECT_NODE):
	  mouse_select_node(event->x, event->y);
	  break;
	}
    }
  update_drawing_area(drawing_area);
  return(TRUE);
}

static gint motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
{
  gint x, y;
  GdkModifierType state;

  if(mode!=MODE_DRAG_NODE)
    return(FALSE);
  
  if(event->is_hint)
    gdk_window_get_pointer(event->window, &x, &y, &state);
  else
    {
      x=event->x;
      y=event->y;
      state=event->state;
    }
  if(state & GDK_BUTTON1_MASK && pixmap !=NULL)
    {
      mouse_drag_node(x, y);
      update_drawing_area(widget);
    }
  return(TRUE);

}

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event)
{
  if(pixmap)
    gdk_pixmap_unref(pixmap);
  pixmap=gdk_pixmap_new(widget->window, widget->allocation.width,
			widget->allocation.height, -1);
  update_drawing_area(widget);
  return(TRUE);
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		  pixmap, event->area.x, event->area.y, event->area.x, event->area.y,
		  event->area.width, event->area.height);
  return(FALSE);
}

void quit()
{
  gtk_exit(0);
}

void update_wave_title()
{
  gtk_label_set_text(GTK_LABEL(waveform_title_label), waveform_title());
}

void update_freq_text(long freq)
{
  gchar *text=(gchar *) calloc(6, sizeof(char));
  sprintf(text, "%ld", freq);
  if(freq_entrybox!=NULL)
    gtk_entry_set_text(GTK_ENTRY(freq_entrybox), text);   
}

void update_mixing_text(double mixing)
{
  gchar *text=(gchar *) calloc(6, sizeof(char));
  sprintf(text, "%.3f", mixing);
  if(mixing_entrybox!=NULL)
    gtk_entry_set_text(GTK_ENTRY(mixing_entrybox), text);   
}

void update_entry_text(long length)
{
  gchar *text=(gchar *) calloc(6, sizeof(char));
  sprintf(text, "%ld", length);
  if(sample_length_entrybox!=NULL)
    gtk_entry_set_text(GTK_ENTRY(sample_length_entrybox), text);   
}

void update_entryboxs()
{
  char title[80];
  if(filename!=NULL)
    {
      sprintf(title, "Waveform Editor - %s", filename);
      gtk_window_set_title (GTK_WINDOW (window), title);
    }
  else
    {
      gtk_window_set_title (GTK_WINDOW (window), "Waveform Editor - Untitled");
    }
  update_wave_title();
  update_entry_text(current_waveform->sample_length);
  update_mixing_text(current_waveform->mixing_coefficient);
  update_freq_text(current_waveform->freq);
}

void add_new_waveform(char *title, int type, long length, double mixing, glong freq)
{
  FREQ_AMPLITUDE_ENTRY *new_entry;

  new_entry=create_new_waveform(title, type, length, mixing, freq);
  new_entry->include_in_output=TRUE;
  waveforms=g_list_append(waveforms, new_entry);
  waveform_index=g_list_length(waveforms); 
}

static GtkItemFactoryEntry menu_items[] = {
  { "/_File", NULL, NULL, 0, "<Branch>" },
  { "/File/_New", "<control>N", new_callback, 0, NULL},
  { "/File/_Open", "<control>O", open_callback, 0, NULL},
  { "/File/_Save", "<control>S", save_callback, 0, NULL},
  { "/File/_Play waveform", "<control>P", play_button_callback, 0, NULL},
  { "/File/Output to wav", "<control>O", output_wav_callback, 0, NULL},
  { "/File/Save _As", "<control>A", save_as_callback, 0, NULL},
  { "/File/E_xit", "<control>X", exit_menu_item, 0, NULL},
  { "/_Options", NULL, NULL, 0, "<Branch>" },
  { "/Options/Change sound sample se_ttings", "<control>T", NULL, 0, NULL},
  { "/Options/Change sample _length", "<control>L", NULL, 0, NULL},
  { "/Options/Change current waveform _settings", "<control>W", NULL, 0, NULL},
  { "/Options/Half-cos interpolation", "<control>H", half_cos_callback, 0, "<ToggleItem>"},
  { "/Options/_Prefrences", "<control>P", NULL, 0, NULL},
  { "/_Tools", NULL, NULL, 0, "<Branch>" },
  { "/Tools/Add _copy of current waveform", "<control>C", copy_waveform_callback, 0, NULL},
  { "/Tools/_Normalize nodes", "<control>N", normalize_nodes_callback, 0, NULL},
  { "/_Help", NULL, NULL, 0, "<Branch>" },
  { "/Help/_Manual",  "<control>M", NULL, 0, NULL},
  { "/Help/_About",  "<control>A", NULL, 0, NULL},
};

void get_main_menu(GtkWidget *window, GtkWidget **menubar)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items=sizeof(menu_items) / sizeof(menu_items[0]);

  accel_group=gtk_accel_group_new();
  item_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
  gtk_window_add_accel_group(GTK_WINDOW (window), accel_group);

  if(menubar)
    *menubar=gtk_item_factory_get_widget(item_factory, "<main>");
}

int main(int argc, char *argv[])
{
  GtkWidget *menubar, *main_table, *forward_button, 
    *backward_button, 
    *sample_length_label, *add_radio_button, *play_button,
    *select_radio_button, *delete_radio_button, *drag_radio_button, 
    *set_sample_length_button, *add_new_waveform_button,
    *delete_this_waveform_button, *play_only_current_button, 
    *mixing_entrybox_label, *mixing_entrybox_button, *freq_entrybox_button,
    *freq_entrybox_label;
  char *waveform_title_label_text;
  GList *temp_glist;

  waveforms=NULL;
  sample_length_entrybox=NULL;
  default_length=DEFAULT_SAMPLE_LENGTH;
  filename=NULL;

  generate_wavetable();

  /* add basic first waveform */

  add_new_waveform("Untitled 1", TYPE_WAVE, DEFAULT_SAMPLE_LENGTH, 1, DEFAULT_FREQ);
  temp_glist=g_list_nth(waveforms, 0);
  waveform_index=0;
  interpolation=I_LINEAR;
  current_waveform=(FREQ_AMPLITUDE_ENTRY *) temp_glist->data;
  waveform_title_label_text=waveform_title();
  mode=MODE_ADD_NODE;

  gtk_init(&argc, &argv);

  window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy", 
		     GTK_SIGNAL_FUNC(gtk_main_quit),
		     "WM destroy");
  gtk_window_set_title(GTK_WINDOW(window), "Waveform Editor");
  gtk_widget_set_usize(GTK_WIDGET(window), 500, 400);

  main_table=gtk_table_new(3, 10, FALSE);
  gtk_container_add(GTK_CONTAINER(window), main_table);
  gtk_widget_show(main_table);

  get_main_menu(window, &menubar);
  gtk_table_attach_defaults(GTK_TABLE(main_table), menubar, 0, 3, 0, 1);
  gtk_widget_show(menubar);

  waveform_title_label=gtk_label_new(waveform_title_label_text);
  gtk_table_attach_defaults(GTK_TABLE(main_table), waveform_title_label, 0, 1, 1, 2);
  gtk_widget_show(waveform_title_label);

  drawing_area=gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), 390, 200);
  gtk_table_attach_defaults(GTK_TABLE(main_table), drawing_area, 0, 3, 2, 3);
  gtk_widget_show(drawing_area);

  gtk_signal_connect(GTK_OBJECT(drawing_area), "expose_event", (GtkSignalFunc) 
		     expose_event, NULL);
  gtk_signal_connect(GTK_OBJECT(drawing_area), "configure_event", (GtkSignalFunc) 
		     configure_event, NULL);
  gtk_signal_connect(GTK_OBJECT(drawing_area), "button_press_event", (GtkSignalFunc)
		     button_press_event, NULL);
  gtk_signal_connect(GTK_OBJECT(drawing_area), "motion_notify_event", (GtkSignalFunc)
		     motion_notify_event, NULL);
  gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK
			| GDK_POINTER_MOTION_MASK);

  backward_button=gtk_button_new_with_label("Previous Waveform");
  gtk_signal_connect(GTK_OBJECT(backward_button), "clicked",
	     GTK_SIGNAL_FUNC(backward_button_callback), (gpointer) "backward");  
  gtk_table_attach_defaults(GTK_TABLE(main_table), backward_button, 0, 1, 3, 4);
  gtk_widget_show(backward_button);

  forward_button=gtk_button_new_with_label("Next Waveform");
  gtk_signal_connect (GTK_OBJECT(forward_button), "clicked",
	     GTK_SIGNAL_FUNC(forward_button_callback), (gpointer) "forward");
  gtk_table_attach_defaults(GTK_TABLE(main_table), forward_button, 1, 2, 3, 4);
  gtk_widget_show(forward_button);

  include_this_toggle=gtk_check_button_new_with_label("Include this waveform in output:");
  gtk_signal_connect(GTK_OBJECT(include_this_toggle), "clicked",
	     GTK_SIGNAL_FUNC(include_this_toggle_callback), (gpointer) "include");
  gtk_table_attach_defaults(GTK_TABLE(main_table), include_this_toggle, 2, 3, 3, 4);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(include_this_toggle), TRUE);
  gtk_widget_show(include_this_toggle);
   
  sample_length_label=gtk_label_new("Enter sample length:");
  gtk_table_attach_defaults(GTK_TABLE(main_table), sample_length_label, 0, 1, 4, 5);
  gtk_widget_show(sample_length_label);

  sample_length_entrybox=gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(main_table), sample_length_entrybox, 1, 2, 4, 5);
  gtk_entry_set_text(GTK_ENTRY(sample_length_entrybox), initial_sample_length_text);
  gtk_widget_show(sample_length_entrybox);

  set_sample_length_button=gtk_button_new_with_label("Set Sample Length");
  gtk_signal_connect(GTK_OBJECT( set_sample_length_button), "clicked",
	  GTK_SIGNAL_FUNC(set_sample_length_button_callback), 
		      (gpointer) "sample");
  gtk_table_attach_defaults(GTK_TABLE(main_table), set_sample_length_button,
			    2, 3, 4, 5);
  gtk_widget_show(set_sample_length_button);

  mixing_entrybox_label=gtk_label_new("Enter mixing coefficent:");
  gtk_table_attach_defaults(GTK_TABLE(main_table), mixing_entrybox_label, 0, 1, 5, 6);
  gtk_widget_show(mixing_entrybox_label);

  mixing_entrybox=gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(main_table), mixing_entrybox, 1, 2, 5, 6);
  gtk_entry_set_text(GTK_ENTRY(sample_length_entrybox), "1");
  gtk_widget_show(mixing_entrybox);

  mixing_entrybox_button=gtk_button_new_with_label("Set Mixing Coefficient");
  gtk_table_attach_defaults(GTK_TABLE(main_table), mixing_entrybox_button,
			    2, 3, 5, 6);

  gtk_signal_connect (GTK_OBJECT(mixing_entrybox_button), "clicked",
	  GTK_SIGNAL_FUNC(mixing_entrybox_button_callback), 
		      (gpointer) "sample");
  gtk_widget_show(mixing_entrybox_button);  


  freq_entrybox_label=gtk_label_new("Enter frequency:");
  gtk_table_attach_defaults(GTK_TABLE(main_table), freq_entrybox_label, 0, 1, 6, 7);
  gtk_widget_show(freq_entrybox_label);

  freq_entrybox=gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(main_table), freq_entrybox, 1, 2, 6, 7);
  gtk_entry_set_text(GTK_ENTRY(freq_entrybox), "1000");
  gtk_widget_show(freq_entrybox);

  freq_entrybox_button=gtk_button_new_with_label("Set Frequency");
  gtk_table_attach_defaults(GTK_TABLE(main_table), freq_entrybox_button,
			    2, 3, 6, 7);

  gtk_signal_connect (GTK_OBJECT(freq_entrybox_button), "clicked",
	  GTK_SIGNAL_FUNC(freq_entrybox_button_callback), 
		      (gpointer) "sample");
  gtk_widget_show(freq_entrybox_button);

  add_radio_button=gtk_radio_button_new_with_label(NULL, "Add Node");
  select_radio_button=gtk_radio_button_new_with_label(gtk_radio_button_group
			 (GTK_RADIO_BUTTON (add_radio_button)), "Select Node");
  delete_radio_button=gtk_radio_button_new_with_label(gtk_radio_button_group
			 (GTK_RADIO_BUTTON (select_radio_button)), "Delete Node");
  drag_radio_button=gtk_radio_button_new_with_label(gtk_radio_button_group
			 (GTK_RADIO_BUTTON (delete_radio_button)), "Drag Node");

  gtk_table_attach_defaults(GTK_TABLE(main_table), add_radio_button, 0, 1, 7, 8);
  gtk_signal_connect (GTK_OBJECT(add_radio_button), "clicked",
	     GTK_SIGNAL_FUNC(add_radio_button_callback), (gpointer) "1");
  gtk_widget_show(add_radio_button);


  gtk_table_attach_defaults(GTK_TABLE(main_table), delete_radio_button, 1, 2, 7, 8);
  gtk_signal_connect(GTK_OBJECT(delete_radio_button), "clicked",
	     GTK_SIGNAL_FUNC(delete_radio_button_callback), (gpointer) "3");
  gtk_widget_show(delete_radio_button);

  gtk_table_attach_defaults(GTK_TABLE(main_table), drag_radio_button, 2, 3, 7, 8);
  gtk_signal_connect(GTK_OBJECT(drag_radio_button), "clicked",
	     GTK_SIGNAL_FUNC(drag_radio_button_callback), (gpointer) "4");
  gtk_widget_show(drag_radio_button);

  play_button=gtk_button_new_with_label("Play waveform");
  gtk_table_attach_defaults(GTK_TABLE(main_table), play_button, 0, 3, 8, 9);
  gtk_signal_connect(GTK_OBJECT(play_button), "clicked",
		     GTK_SIGNAL_FUNC(play_button_callback), (gpointer) "play");
  gtk_widget_show(play_button);

  add_new_waveform_button=gtk_button_new_with_label("Add new waveform");  
  gtk_table_attach_defaults(GTK_TABLE(main_table), add_new_waveform_button, 0, 1, 9, 10);
  gtk_signal_connect(GTK_OBJECT(add_new_waveform_button), "clicked",
	     GTK_SIGNAL_FUNC(add_new_waveform_button_callback), (gpointer) "add");
  gtk_widget_show(add_new_waveform_button);

  delete_this_waveform_button=gtk_button_new_with_label("Delete current waveform");  
  gtk_table_attach_defaults(GTK_TABLE(main_table), delete_this_waveform_button, 1, 2, 9, 10);
  gtk_signal_connect(GTK_OBJECT(delete_this_waveform_button), "clicked",
       GTK_SIGNAL_FUNC(delete_current_waveform_button_callback), (gpointer) "delete_current");
  gtk_widget_show(delete_this_waveform_button);

  play_only_current_button=gtk_button_new_with_label("Play only current");  
  gtk_table_attach_defaults(GTK_TABLE(main_table), play_only_current_button, 2, 3, 9, 10);
  gtk_signal_connect(GTK_OBJECT(play_only_current_button), "clicked",
	       GTK_SIGNAL_FUNC(play_only_current_button_callback), (gpointer) "play_current");
  gtk_widget_show(play_only_current_button);

  gtk_widget_show(window);

  update_entryboxs();
  gtk_main();
  return(0);
}





