#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <audiofile.h>
#include <gtk/gtk.h>
#include <strings.h>
#include <string.h>
#include <math.h>

#define MODE_ADD_NODE 0
#define MODE_DELETE_NODE 1
#define MODE_SELECT_NODE 2
#define MODE_DRAG_NODE 3

#define I_LINEAR 0
#define I_HALF_COS 1

#define DEFAULT_FREQ 1000
#define DEFAULT_SAMPLE_LENGTH 44100
#define DEFAULT_ENVELOPE_LENGTH 44100
#define BUFFER_LENGTH 44100
#define LOWEST_PERCEIVABLE_FREQ 20
#define SAMPLES_PER_SEC 44100

#define WAVETABLE_LENGTH SAMPLES_PER_SEC/LOWEST_PERCEIVABLE_FREQ+1

#define TYPE_ENVELOPE 0
#define TYPE_WAVE     1

#define MAX_AMPLITUDE 32767
#define PI 3.1415926F

typedef struct LINE
{
  gdouble m;
  glong b;
} LINE;

typedef struct COORDINATES
{
  glong x_pos;
  glong amplitude;
} COORDINATES;

typedef struct GRAPH_POINTS
{
  COORDINATES *coords;
  glong length;
} GRAPH_POINTS;

typedef struct SAMPLE
{
  short int *data;
  long length;
  int type;
} SAMPLE;

typedef struct FREQ_AMPLITUDE_ENTRY
{
  char *title;
  int type;
  glong index;
  glong freq;
  GList **nodes;
  gint include_in_output;
  double mixing_coefficient;
  glong sample_length;

  gint changed;
  SAMPLE *samp;
} FREQ_AMPLITUDE_ENTRY;

extern void free_sample(SAMPLE *samp);
extern void free_nodes(GList **nodes);
extern void normalize_nodes(GList *waveforms);
extern void free_waveform_glist(GList *waveforms);
extern SAMPLE *combine_samples(SAMPLE **sample_array, int number_of_samples);
extern GList *open_wave(char *filename);
extern void update_entryboxs();
extern gint output_as_wav(SAMPLE *a_sample, char *filename);
extern gint save_wave(char *filename, GList *waveforms);
extern void update_entry_text(long length);
extern void update_mixing_text(double mixing);
extern void redo_xpos(FREQ_AMPLITUDE_ENTRY *waveform, long length);
extern long largest_xpos_in_entries(GList *waveforms);
extern void apply_all_envelopes(GList *list, SAMPLE *the_sample);
extern gint count_wave_nodes(GList *waveforms);
extern GRAPH_POINTS get_graph_points(FREQ_AMPLITUDE_ENTRY *waveform, long window_width, long window_height);
extern FREQ_AMPLITUDE_ENTRY *create_new_waveform(char *title, int type, long length, double mixing, glong freq);
extern void add_node(GList **n, COORDINATES x);
extern COORDINATES mouse_coords_to_wave_coords(gint x, gint y, long sample_length, long window_width, long window_height);
extern void add_new_waveform(char *title, int type, long length, double mixing, glong freq);
extern void amplify_sample(SAMPLE *x, double coefficient);
extern gint play_sample(SAMPLE *samp);
extern gint does_xpos_exist(GList **header, long x);
extern gint delete_xpos(GList **header, long x);
extern SAMPLE *build_sample(FREQ_AMPLITUDE_ENTRY *waveform, gint interpolation_mode);
extern SAMPLE *create_sound(GList *waveforms, int interpolation_method);
extern long largest_xpos(GList **);
extern void apply_all_envelopes(GList *list, SAMPLE *the_sample);
extern gint apply_envelope(GList **list, SAMPLE *a_sample);
extern FREQ_AMPLITUDE_ENTRY *copy_waveform(FREQ_AMPLITUDE_ENTRY *src);
extern void generate_wavetable();

